@@ -184,18 +184,14 @@ message_free (Message *message)
 {
 	g_free (message->method);
 	g_free (message->object_path);
-	
-	g_list_foreach (message->listeners, (GFunc)listener_free, NULL);
-	g_list_free (message->listeners);
-	
+	g_list_free_full (message->listeners, (GDestroyNotify) listener_free);
 	g_free (message);
 }
 
 static void
 message_queue_free (GList *queue)
 {
-	g_list_foreach (queue, (GFunc)g_object_unref, NULL);
-	g_list_free (queue);
+	g_list_free_full (queue, g_object_unref);
 }
 
 static void
@@ -545,7 +541,7 @@ gedit_message_bus_init (GeditMessageBus *self)
  *
  * Get the default application #GeditMessageBus.
  *
- * Return value: (transfer full): the default #GeditMessageBus
+ * Return value: (transfer none): the default #GeditMessageBus
  *
  */
 GeditMessageBus *
@@ -641,7 +637,6 @@ gedit_message_bus_register (GeditMessageBus *bus,
 			    ...)
 {
 	gchar *identifier;
-	gpointer data;
 	va_list var_args;
 	GeditMessageType *message_type;
 
@@ -655,7 +650,6 @@ gedit_message_bus_register (GeditMessageBus *bus,
 	}
 
 	identifier = gedit_message_type_identifier (object_path, method);
-	data = g_hash_table_lookup (bus->priv->types, identifier);
 	
 	va_start (var_args, num_optional);
 	message_type = gedit_message_type_new_valist (object_path, 
@@ -1179,4 +1173,4 @@ gedit_message_bus_send_sync (GeditMessageBus *bus,
 	return message;
 }
 
-/* ex:ts=8:noet: */
+/* ex:set ts=8 noet: */
