@@ -17,11 +17,7 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 import os
-import gtk
-from gtk import gdk
-import gio
-import gedit
-#import gtksourceview
+from gi.repository import Gio, Gtk, Gdk, GtkSource, Gedit
 from outputpanel import OutputPanel
 from capture import *
 
@@ -35,16 +31,16 @@ def current_word(document):
     piter = document.get_iter_at_mark(document.get_insert())
     start = piter.copy()
     
-    if not start.starts_word():
+    if not piter.starts_word() and (piter.inside_word() or piter.ends_word()):
         start.backward_word_start()
     
-    if not piter.ends_word():
+    if not piter.ends_word() and piter.inside_word():
         piter.forward_word_end()
             
     return (start, piter)
 
 # ==== Capture related functions ====
-def run_external_tool(window, node):
+def run_external_tool(window, panel, node):
     # Configure capture environment
     try:
         cwd = os.getcwd()
@@ -59,7 +55,7 @@ def run_external_tool(window, node):
     if view is not None:
         # Environment vars relative to current document
         document = view.get_buffer()
-        uri = document.get_uri()
+        location = document.get_location()
         
         # Current line number
         piter = document.get_iter_at_mark(document.get_insert())
@@ -86,37 +82,43 @@ def run_external_tool(window, node):
         
         capture.set_env(GEDIT_CURRENT_DOCUMENT_TYPE=document.get_mime_type())
         
-        if uri is not None:
-            gfile = gio.File(uri)
-            scheme = gfile.get_uri_scheme()
-            name = os.path.basename(uri)
-            capture.set_env(GEDIT_CURRENT_DOCUMENT_URI    = uri,
+        if location is not None:
+            scheme = location.get_uri_scheme()
+            name = location.get_basename()
+            capture.set_env(GEDIT_CURRENT_DOCUMENT_URI    = location.get_uri(),
                             GEDIT_CURRENT_DOCUMENT_NAME   = name,
                             GEDIT_CURRENT_DOCUMENT_SCHEME = scheme)
-            if gedit.utils.uri_has_file_scheme(uri):
-                path = gfile.get_path()
+            if Gedit.utils.location_has_file_scheme(location):
+                path = location.get_path()
                 cwd = os.path.dirname(path)
                 capture.set_cwd(cwd)
                 capture.set_env(GEDIT_CURRENT_DOCUMENT_PATH = path,
                                 GEDIT_CURRENT_DOCUMENT_DIR  = cwd)
 
-        documents_uri = [doc.get_uri()
-                                 for doc in window.get_documents()
-                                 if doc.get_uri() is not None]
-        documents_path = [gio.File(uri).get_path()
-                                 for uri in documents_uri
-                                 if gedit.utils.uri_has_file_scheme(uri)]
+        documents_location = [doc.get_location()
+                              for doc in window.get_documents()
+                              if doc.get_location() is not None]
+        documents_uri = [location.get_uri()
+                         for location in documents_location
+                         if location.get_uri() is not None]
+        documents_path = [location.get_path()
+                          for location in documents_location
+                          if Gedit.utils.location_has_file_scheme(location)]
         capture.set_env(GEDIT_DOCUMENTS_URI  = ' '.join(documents_uri),
                         GEDIT_DOCUMENTS_PATH = ' '.join(documents_path))
 
-    capture.set_flags(capture.CAPTURE_BOTH)
+    flags = capture.CAPTURE_BOTH
+    
+    if not node.has_hash_bang():
+        flags |= capture.CAPTURE_NEEDS_SHELL
+
+    capture.set_flags(flags)
 
     # Get input text
     input_type = node.input
     output_type = node.output
 
-    # Get the panel
-    panel = window.get_data("ExternalToolsPluginWindowData")._output_buffer
+    # Clear the panel
     panel.clear()
 
     if output_type == 'output-panel':
@@ -162,7 +164,7 @@ def run_external_tool(window, node):
             if not end.ends_word():
                 end.forward_word_end()
 
-        input_text = document.get_text(start, end)
+        input_text = document.get_text(start, end, False)
         capture.set_input(input_text)
 
     # Assign the standard output to the chosen "file"
@@ -206,8 +208,9 @@ def run_external_tool(window, node):
         document.end_user_action()
 
 class MultipleDocumentsSaver:
-    def __init__(self, window, docs, node):
+    def __init__(self, window, panel, docs, node):
         self._window = window
+        self._panel = panel
         self._node = node
         self._error = False
 
@@ -219,7 +222,7 @@ class MultipleDocumentsSaver:
 
         for doc in docs:
             signals[doc] = doc.connect('saving', self.on_document_saving)
-            gedit.commands.save_document(window, doc)
+            Gedit.commands.save_document(window, doc)
             doc.disconnect(signals[doc])
     
     def on_document_saving(self, doc, size, total_size):
@@ -236,17 +239,17 @@ class MultipleDocumentsSaver:
         self._counter -= 1
         
         if self._counter == 0 and not self._error:
-            run_external_tool(self._window, self._node)
+            run_external_tool(self._window, self._panel, self._node)
 
-def capture_menu_action(action, window, node):
+def capture_menu_action(action, window, panel, node):
     if node.save_files == 'document' and window.get_active_document():
-        MultipleDocumentsSaver(window, [window.get_active_document()], node)
+        MultipleDocumentsSaver(window, panel, [window.get_active_document()], node)
         return
     elif node.save_files == 'all':
-        MultipleDocumentsSaver(window, window.get_documents(), node)
+        MultipleDocumentsSaver(window, panel, window.get_documents(), node)
         return
 
-    run_external_tool(window, node)
+    run_external_tool(window, panel, node)
 
 def capture_stderr_line_panel(capture, line, panel):
     if not panel.visible():
@@ -255,7 +258,7 @@ def capture_stderr_line_panel(capture, line, panel):
     panel.write(line, panel.error_tag)
 
 def capture_begin_execute_panel(capture, panel, view, label):
-    view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gdk.Cursor(gdk.WATCH))
+    view.get_window(Gtk.TextWindowType.TEXT).set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
 
     panel['stop'].set_sensitive(True)
     panel.clear()
@@ -270,16 +273,20 @@ def capture_end_execute_panel(capture, exit_code, panel, view, output_type):
         start = doc.get_start_iter()
         end = start.copy()
         end.forward_chars(300)
+        uri = ''
 
-        mtype = gio.content_type_guess(data=doc.get_text(start, end))
-        lmanager = gedit.get_language_manager()
-        
-        language = lmanager.guess_language(doc.get_uri(), mtype)
+        mtype, uncertain = Gio.content_type_guess(None, doc.get_text(start, end, False))
+        lmanager = GtkSource.LanguageManager.get_default()
+
+        location = doc.get_location()
+        if location:
+            uri = location.get_uri()
+        language = lmanager.guess_language(uri, mtype)
         
         if language is not None:
             doc.set_language(language)
 
-    view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gdk.Cursor(gdk.XTERM))
+    view.get_window(Gtk.TextWindowType.TEXT).set_cursor(Gdk.Cursor.new(Gdk.CursorType.XTERM))
     view.set_cursor_visible(True)
     view.set_editable(True)
 
